/*====================================+=====================================+
! File CDlgAddLive.cpp                ! Copyright (C) 2016-2019 Remi PASCAL !
+-------------------------------------+-------------------------------------+
! This file is part of matelefree.                                          !
! matelefree is free software: you can redistribute it and/or modify it     !
! under the terms of the GNU General Public License as published by the     !
! Free Software Foundation, either version 3 of the License, or any later   !
! version.                                                                  !
! matelefree is distributed in the hope that it will be useful, but WITHOUT !
! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or     !
! FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for !
! more details.                                                             !
! You should have received a copy of the GNU General Public License along   !
! with matelefree. If not, see <http://www.gnu.org/licenses/>.              !
+==========================================================================*/



/*-------------------------------------------------------------------------*/
#include <wx/uri.h>
#include <wx/wfstream.h>
#include <wx/txtstrm.h>
#include <wx/mstream.h>
#include <wx/dir.h>
#include "CApp.h"
#include "CChannel.h"
#include "CDlgAddLive.h"
/*-------------------------------------------------------------------------*/



/*--( The column will be created outside of the constructor )--------------*/
CLiveList::CLiveList( CDlgAddLive *dlg, CPlaylistLive *pl )
         : wxListCtrl( dlg, wxID_ANY, wxDefaultPosition, wxDefaultSize,
                       wxLC_REPORT | wxLC_VIRTUAL | wxLC_NO_HEADER
                     ),
           m_dlg( dlg ),
           m_pl( pl )
{
   /*----------------------------------------------------------------------*/
   Bind( wxEVT_LIST_ITEM_ACTIVATED,
         [ this ]( wxListEvent &event )
         {  add_channel( event.GetIndex() ) ; }
       ) ;
   /*----------------------------------------------------------------------*/
   int i_col_width_name = 0 ;

   /*-----------------------------------------------------------------------+
   ! Load the live channels list proposed to be added : it can be split     !
   ! into different files.                                                  !
   ! They must all begin with the same origin.                              !
   ! For example : twitch.txt, twitch_2.txt, twitchB.txt ...                !
   +-----------------------------------------------------------------------*/
   wxArrayString as_live_list ;
   /*----------------------------------------------------------------------*/
   wxDir::GetAllFiles( wxGetApp().m_s_media_path + "live", &as_live_list,
                       m_pl->get_source() + "*.txt",
                       wxDIR_FILES | wxDIR_DIRS
                     ) ;
   /*----------------------------------------------------------------------*/
   for( const auto &s_fn : as_live_list )
   {
      /*-------------------------------------------------------------------*/
      wxLogNull no_log ; // Suppress useless warnings
      wxFileInputStream fis_live( s_fn ) ;
      /*-------------------------------------------------------------------*/
      if( fis_live.IsOk() )
      {
         /*--( Using specific conv class to handle UTF8 / 8859-1 files )---*/
         wxTextInputStream tis_live( fis_live, "", CConv_UTF8_8859_1() ) ;
         wxString s_line ;
         str_live_info info ;
         int i_line_num = 0 ;
         /*----------------------------------------------------------------*/
         while( !tis_live.GetInputStream().Eof() )
         {
            /*--------------------------------------------------------------+
            ! Read and parse line                                           !
            ! Empty lines & lines starting with '#' are seen as comments    !
            ! Each line is made of 2 elements separated by one or more      !
            ! spaces : id name                                              !
            +--------------------------------------------------------------*/
            s_line = mtf::read_line( tis_live, i_line_num ) ;
            if(    tis_live.GetInputStream().Eof()
                || mtf::match( s_line, R"=(^\s*(.*?)\s+(.*)$)=",
                               &info.m_s_id, &info.m_s_name
                             ) != 3
              )
            {  continue ; }

            /*--( Useless to list a channel if already in the playlist )---*/
            if( m_pl->find_channel( info.m_s_id ) != nullptr )
            {  continue ; }

            /*--( Useless to list a channel more than once )---------------*/
            auto it_li
            = std::find_if( m_vec_live_info.begin(), m_vec_live_info.end(),
                            [ &info ]( str_live_info &info_look )
                            {  return( info.m_s_id == info_look.m_s_id ) ; }
                          ) ;
            if( it_li != m_vec_live_info.end() ) { continue ; }

            /*-------------------------------------------------------------*/
            m_vec_live_info.emplace_back( info ) ;
            /*-------------------------------------------------------------*/
              i_col_width_name
            = wxMax( i_col_width_name,
                     GetTextExtent( info.m_s_name ).GetWidth()
                   ) ;
            /*-------------------------------------------------------------*/
         }
         /*----------------------------------------------------------------*/
      }
      /*-------------------------------------------------------------------*/
   }

   /*--( It seems more user friendly to have channels sorted by name )-----*/
   std::sort( m_vec_live_info.begin(), m_vec_live_info.end(),
              []( const str_live_info &s1, const str_live_info &s2 )
              {  return( s1.m_s_name.CmpNoCase( s2.m_s_name ) < 0 ) ; }
            ) ;

   /*--( Add "margins" )---------------------------------------------------*/
   i_col_width_name += 20 ;
   /*----------------------------------------------------------------------*/
   AppendColumn( wxEmptyString, wxLIST_FORMAT_LEFT, i_col_width_name ) ;
   /*----------------------------------------------------------------------*/
   init_item_count() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
wxString CLiveList::OnGetItemText( long l_item, long l_column ) const
{
   /*----------------------------------------------------------------------*/
   if( l_item < 0 && l_item >= static_cast< int >( m_vec_live_info.size() ) )
   {  return( wxEmptyString ) ; }
   /*----------------------------------------------------------------------*/
   return( m_vec_live_info[ l_item ].m_s_name ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CLiveList::init_item_count()
{
   /*----------------------------------------------------------------------*/
   SetItemCount( m_vec_live_info.size() ) ;
   Refresh() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CLiveList::del_entry( const wxString &s_id )
{
   /*----------------------------------------------------------------------*/
   const size_t co_sz_org = m_vec_live_info.size() ;
   /*----------------------------------------------------------------------*/
   for( int i_num = static_cast< int >( co_sz_org - 1 ); i_num >= 0; --i_num)
   {
      /*-------------------------------------------------------------------*/
      if( m_vec_live_info[ i_num ].m_s_id == s_id )
      {  m_vec_live_info.erase( m_vec_live_info.begin() + i_num ) ; }
      /*-------------------------------------------------------------------*/
   }
   /*--( No change ? )-----------------------------------------------------*/
   if( co_sz_org == m_vec_live_info.size() ) { return ; }
   /*--( The list has changed )--------------------------------------------*/
   init_item_count() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CLiveList::add_channel( int i_item )
{
   /*----------------------------------------------------------------------*/
   str_live_info *p_info = &m_vec_live_info[ i_item ] ;
   /*----------------------------------------------------------------------*/
   CChannelLive *p_channel
              = m_pl->new_channel( p_info->m_s_id, p_info->m_s_name ) ;
   if( m_pl->add_channel( p_channel ) != 0 )
   {  delete p_channel ; }
   else
   {  /*--( After being added the channel has to be removed from the list )*/
      m_vec_live_info.erase( m_vec_live_info.begin() + i_item ) ;
      init_item_count() ;
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
CDlgAddLive::CDlgAddLive( wxWindow *parent, CPlaylistLive *pl )
           : wxDialog( parent, wxID_ANY, wxEmptyString,
                       wxDefaultPosition, wxDefaultSize,
                       wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER
                     ),
             m_pl( pl )
{
   /*----------------------------------------------------------------------*/
   SetTitle( wxString::Format( _( "%s : add channels" ), m_pl->get_label()));
   /*--( Build UI : Live URL part )----------------------------------------*/
   wxBoxSizer *bsz_gen = new wxBoxSizer( wxVERTICAL ) ;
   wxStaticBoxSizer *stbsz_url
      = new wxStaticBoxSizer( wxVERTICAL, this,
                              _( "Add a channel by its URL" )
                            ) ;
   wxFlexGridSizer *fgsz = new wxFlexGridSizer( 3, 0, 0 ) ;
   fgsz->AddGrowableCol( 1 ) ;
   fgsz->Add( new wxStaticText( this, wxID_ANY, _( "Live video URL" ) ),
              0, wxALIGN_CENTER_VERTICAL | wxALL, 5
            ) ;
   m_txt_url = new wxTextCtrl( this, wxID_ANY, m_pl->get_url_example(),
                               wxDefaultPosition, wxDefaultSize,
                               wxTE_PROCESS_ENTER
                             ) ;
   m_txt_url->SetMinSize( wxSize( 300, -1 ) ) ;
   m_txt_url->SetToolTip(   _( "The URL must follow the format :" )
                          + '\n' + m_pl->get_url_example()
                        ) ;

   fgsz->Add( m_txt_url, 1, wxALL | wxEXPAND, 5 ) ;
   wxButton *bt_add_url = new wxButton( this, wxID_ANY, _( "Add" ) ) ;
   fgsz->Add( bt_add_url, 1, wxALL, 5 ) ;

   fgsz->AddStretchSpacer( 1 ) ;
   m_stx_msg = new wxStaticText( this, wxID_ANY, wxEmptyString,
                                 wxDefaultPosition, wxDefaultSize,
                                 wxST_ELLIPSIZE_END
                               ) ;
   fgsz->Add( m_stx_msg, 1, wxALL | wxEXPAND, 5 ) ;
   stbsz_url->Add( fgsz, 0, wxALL | wxEXPAND, 5 ) ;

   bsz_gen->AddSpacer( 15 ) ;
   bsz_gen->Add( stbsz_url, 0, wxALL | wxEXPAND, 5 ) ;

   /*--( Build UI : List part )--------------------------------------------*/
   wxStaticBoxSizer *stbsz_list
     = new wxStaticBoxSizer( wxVERTICAL, this,
                             _( "Add a channel by double-clicking it" )
                           ) ;
   m_list = new CLiveList( this, m_pl ) ;
   stbsz_list->Add( m_list, 1, wxALL | wxEXPAND, 5 ) ;

   bsz_gen->AddSpacer( 15 ) ;
   bsz_gen->Add( stbsz_list, 1, wxALL | wxEXPAND, 5 ) ;

   /*--( Button )----------------------------------------------------------*/
   bsz_gen->AddSpacer( 15 ) ;
   wxButton *bt_close = new wxButton( this, wxID_CLOSE ) ;
   bsz_gen->Add( bt_close, 0, wxALL | wxALIGN_CENTER_HORIZONTAL, 5 ) ;

   SetSizer( bsz_gen ) ;

   /*----------------------------------------------------------------------*/
   m_txt_url->Bind( wxEVT_COMMAND_TEXT_ENTER,
                    &CDlgAddLive::OnURLEnter, this
                  ) ;
   bt_add_url->Bind( wxEVT_COMMAND_BUTTON_CLICKED,
                     &CDlgAddLive::OnURLEnter, this
                   ) ;
   bt_close->Bind( wxEVT_COMMAND_BUTTON_CLICKED,
                   [ this ]( wxCommandEvent &event ) { Close() ; }
                 ) ;
   /*----------------------------------------------------------------------*/
   mtf::init_tlw_rect( this, wxGetApp().M_addlive_rec ) ;
   Layout() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
CDlgAddLive::~CDlgAddLive()
{
   /*----------------------------------------------------------------------*/
   mtf::save_tlw_rect( this, wxGetApp().M_addlive_rec ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CDlgAddLive::OnURLEnter( wxCommandEvent &event )
{
   /*----------------------------------------------------------------------*/
   add_channel_by_url( m_txt_url->GetValue().Trim().Trim( false ) ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CDlgAddLive::msg_error( const wxString &s_msg )
{
   /*----------------------------------------------------------------------*/
   m_stx_msg->SetForegroundColour( *wxRED ) ;
   m_stx_msg->SetLabelText( s_msg ) ;
   m_stx_msg->Update() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CDlgAddLive::msg_info( const wxString &s_msg )
{
   /*----------------------------------------------------------------------*/
   m_stx_msg->SetForegroundColour( *wxBLUE ) ;
   m_stx_msg->SetLabelText( s_msg ) ;
   m_stx_msg->Update() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CDlgAddLive::clear_msg()
{
   /*----------------------------------------------------------------------*/
   m_stx_msg->SetLabelText( wxEmptyString ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
int CDlgAddLive::add_channel_by_url( const wxString &s_url )
{
   /*----------------------------------------------------------------------*/
   if( s_url.empty() )
   {  msg_error( _( "The URL is mandatory" ) ) ;
      return( -1 ) ;
   }
   /*----------------------------------------------------------------------*/
   msg_info( _( "Checking ..." ) ) ;
   /*----------------------------------------------------------------------*/
   wxString s_id   ;
   wxString s_name ;
   wxString s_err_msg = _( "Invalid URL" ) ;
   /*--( Basic "general" check )-------------------------------------------*/
   if( m_pl->extract_data_from_url( s_url, s_id, s_name, &s_err_msg ) != 0 )
   {  msg_error( s_err_msg ) ;
      return( -2 ) ;
   }

   /*--( Already added ? )-------------------------------------------------*/
   CChannel *p_channel_found = m_pl->find_channel( s_id ) ;
   if( p_channel_found != nullptr )
   {  /*-------------------------------------------------------------------*/
      msg_error( wxString::Format( _( "Channel already in the playlist : %s"
                                    ),
                                   p_channel_found->get_short_name()
                                 )
               ) ;
      return( -3 ) ;
      /*-------------------------------------------------------------------*/
   }

   /*--( Some names can contain "html" characters : &#39; )---------------*/
   s_name = mtf::unhtml( s_name ) ;
   CChannelLive *p_channel = m_pl->new_channel( s_id, s_name ) ;

   /*--( If we are here this means the channel can be created )------------*/
   m_pl->add_channel( p_channel ) ;
   /*--( Added to the playlist means not needed anymore in the list )------*/
   m_list->del_entry( p_channel->get_id() ) ;
   msg_info( wxString::Format( _( "Channel added : %s" ),
                               p_channel->get_name()
                             )
           ) ;
   /*----------------------------------------------------------------------*/
   return( 0 ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
int CDlgAddLive::get_modal()
{
   /*----------------------------------------------------------------------*/
   return( ShowModal() ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/



/*==========================================================================+
!                       End of file CDlgAddLive.cpp                         !
+==========================================================================*/
